from graphene import Node, String
from graphene_django import DjangoObjectType

from . import choices, models


class OntologyNode(DjangoObjectType):

    type = String()

    class Meta:
        model = models.Ontology
        interfaces = (Node, )

    def resolve_type(self, info):
        return choices.ONTOLOGY[self.type]


class TermNode(DjangoObjectType):

    source = String()
    term = String()
    url = String()

    class Meta:
        model = models.Term
        interfaces = (Node, )

    def resolve_source(self, info):
        return self.source

    def resolve_term(self, info):
        return self.term

    def resolve_url(self, info):
        return self.url


class SynonymNode(DjangoObjectType):

    scope = String()

    class Meta:
        model = models.Synonym
        interfaces = (Node, )

    def resolve_url(self, info):
        if self.scope:
            return choices.SYNONYM_SCOPES[self.scope]


class CrossReferenceNode(DjangoObjectType):

    class Meta:
        model = models.CrossReference
        interfaces = (Node, )


class RelationshipNode(DjangoObjectType):

    class Meta:
        model = models.Relationship
        interfaces = (Node, )


class RelationshipTypeNode(DjangoObjectType):

    class Meta:
        model = models.RelationshipType
        interfaces = (Node, )



# -*- coding: utf-8 -*-
from django.core.validators import validate_comma_separated_integer_list
from django.db import models
from django.utils.translation import ugettext_lazy as _

from genomix.models import TimeStampedLabelModel
from model_utils.models import TimeStampedModel

from . import choices, managers


class Ontology(TimeStampedModel):
    """Model to keep track of Ontology and version."""

    type = models.PositiveSmallIntegerField(choices=choices.ONTOLOGY)
    label = models.CharField(max_length=100)
    active = models.BooleanField(default=True)

    class Meta:
        verbose_name = _('Ontology')
        verbose_name_plural = _('Ontology')

    def __str__(self):
        return '{0} {1}'.format(
            self.get_type_display(),
            self.label
        )


class Term(models.Model):
    """HPO Terms.
    Source: http://purl.obolibrary.org/obo/hp.obo
    """

    ontology = models.ForeignKey('phenotype_ontologies.Ontology', on_delete=models.CASCADE)
    identifier = models.CharField(max_length=25, db_index=True)
    label = models.CharField(max_length=255, db_index=True)
    description = models.TextField(blank=True)
    alternate_ids = models.CharField(
        max_length=2500,
        blank=True,
        validators=[validate_comma_separated_integer_list]
    )
    created_by = models.CharField(max_length=50, blank=True)
    created = models.CharField(max_length=25, blank=True)
    modified = models.DateTimeField(auto_now=True)

    objects = managers.TermQuerySet.as_manager()

    class Meta:
        verbose_name = _('Term')
        verbose_name_plural = _('Terms')
        index_together = [
            ['ontology', 'identifier']
        ]

    def __str__(self):
        return self.term

    @property
    def source(self):
        return self.ontology.get_type_display()

    @property
    def term(self):
        return '{0}:{1}'.format(
            self.source,
            self.identifier,
        )

    @property
    def url(self):
        if self.source == 'MONDO':
            return 'https://monarchinitiative.org/disease/{0}'.format(self.term)
        elif self.source == 'HP':
            return 'http://compbio.charite.de/hpoweb/showterm?id={0}'.format(self.term)
        elif self.source == 'NCIT':
            return 'http://purl.obolibrary.org/obo/{0}_{1}'.format(
                self.source,
                self.identifier,
            )


class Synonym(TimeStampedModel):
    """Synonms for HPO terms."""

    term = models.ForeignKey(
        'phenotype_ontologies.Term',
        related_name='synonyms',
        on_delete=models.CASCADE,
    )
    description = models.TextField(blank=True)
    scope = models.PositiveSmallIntegerField(choices=choices.SYNONYM_SCOPES, blank=True, null=True)

    objects = managers.SynonymQuerySet.as_manager()

    class Meta:
        verbose_name = _('Synonym')
        verbose_name_plural = _('Synonyms')

    def __str__(self):
        return str(self.term)


class CrossReference(TimeStampedModel):
    """XRefs for HPO terms."""

    term = models.ForeignKey(
        'phenotype_ontologies.Term',
        related_name='xrefs',
        on_delete=models.CASCADE,
    )
    source = models.CharField(max_length=25, db_index=True)
    source_value = models.CharField(max_length=255)

    objects = managers.CrossReferenceQuerySet.as_manager()

    class Meta:
        verbose_name = _('Cross Reference')
        verbose_name_plural = _('Cross References')

    def __str__(self):
        return str(self.term)


class Relationship(TimeStampedModel):
    """Relationship for HPO terms."""

    type = models.ForeignKey(
        'phenotype_ontologies.RelationshipType',
        related_name='relationships',
        on_delete=models.CASCADE,
    )
    term = models.ForeignKey(
        'phenotype_ontologies.Term',
        related_name='relationships',
        on_delete=models.CASCADE,
    )
    related_term = models.ForeignKey(
        'phenotype_ontologies.Term',
        related_name='relationships_related',
        on_delete=models.CASCADE,
    )

    objects = managers.RelationshipQuerySet.as_manager()

    class Meta:
        verbose_name = _('Relationship')
        verbose_name_plural = _('Relationships')

    def __str__(self):
        return str(self.term)


class RelationshipType(TimeStampedLabelModel):

    class Meta:
        verbose_name = _('Relationship Types')
        verbose_name_plural = _('Relationship Types')



# -*- coding: utf-8 -*-
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets
from rest_framework.filters import SearchFilter


from . import filters, models, serializers


class TermViewSet(viewsets.ReadOnlyModelViewSet):
    """ViewSet for viewing HPO Terms."""

    queryset = models.Term.objects.fast()
    serializer_class = serializers.TermSerializer
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filter_class = filters.TermFilter
    search_fields = (
        'label',
        'description',
        'synonyms__description',
    )


class CrossReferenceViewSet(viewsets.ReadOnlyModelViewSet):
    """ViewSet for viewing HPO Term Cross References."""

    queryset = models.CrossReference.objects.fast()
    serializer_class = serializers.CrossReferenceSerializer
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filter_class = filters.CrossReferenceFilter
    search_fields = ('source', 'source_value', )



# Generated by Django 2.0.4 on 2018-04-10 15:22

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import re


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CrossReference',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('source', models.CharField(db_index=True, max_length=25)),
                ('source_value', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name_plural': 'Cross References',
                'verbose_name': 'Cross Reference',
            },
        ),
        migrations.CreateModel(
            name='Ontology',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('type', models.PositiveSmallIntegerField(choices=[(1, 'HP'), (2, 'MONDO'), (3, 'NCIT')])),
                ('label', models.CharField(max_length=100)),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name_plural': 'Ontology',
                'verbose_name': 'Ontology',
            },
        ),
        migrations.CreateModel(
            name='Relationship',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
            ],
            options={
                'verbose_name_plural': 'Relationships',
                'verbose_name': 'Relationship',
            },
        ),
        migrations.CreateModel(
            name='RelationshipType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=50, unique=True)),
                ('description', models.CharField(blank=True, max_length=255)),
                ('slug', models.SlugField(unique=True)),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name_plural': 'Relationship Types',
                'verbose_name': 'Relationship Types',
            },
        ),
        migrations.CreateModel(
            name='Synonym',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('description', models.TextField(blank=True)),
                ('scope', models.PositiveSmallIntegerField(blank=True, choices=[(1, 'EXACT'), (2, 'BROAD'), (3, 'NARROW'), (4, 'RELATED'), (5, 'ABBREVATION')], null=True)),
            ],
            options={
                'verbose_name_plural': 'Synonyms',
                'verbose_name': 'Synonym',
            },
        ),
        migrations.CreateModel(
            name='Term',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(db_index=True, max_length=25)),
                ('label', models.CharField(db_index=True, max_length=255)),
                ('description', models.TextField(blank=True)),
                ('alternate_ids', models.CharField(blank=True, max_length=2500, validators=[django.core.validators.RegexValidator(re.compile('^\\d+(?:\\,\\d+)*\\Z'), code='invalid', message='Enter only digits separated by commas.')])),
                ('created_by', models.CharField(blank=True, max_length=50)),
                ('created', models.CharField(blank=True, max_length=25)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('ontology', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='phenotype_ontologies.Ontology')),
            ],
            options={
                'verbose_name_plural': 'Terms',
                'verbose_name': 'Term',
            },
        ),
        migrations.AddField(
            model_name='synonym',
            name='term',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='synonyms', to='phenotype_ontologies.Term'),
        ),
        migrations.AddField(
            model_name='relationship',
            name='related_term',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relationships_related', to='phenotype_ontologies.Term'),
        ),
        migrations.AddField(
            model_name='relationship',
            name='term',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relationships', to='phenotype_ontologies.Term'),
        ),
        migrations.AddField(
            model_name='relationship',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relationships', to='phenotype_ontologies.RelationshipType'),
        ),
        migrations.AddField(
            model_name='crossreference',
            name='term',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='xrefs', to='phenotype_ontologies.Term'),
        ),
        migrations.AlterIndexTogether(
            name='term',
            index_together={('ontology', 'identifier')},
        ),
    ]



# -*- coding: utf-8
from django.contrib import admin

from . import models


class OntologyAdmin(admin.ModelAdmin):
    model = models.Ontology
    list_display = ('type', 'label', 'created', 'modified')
    search_fields = ('label', )
    save_as = True


class TermAdmin(admin.ModelAdmin):
    model = models.Term
    list_display = ('ontology', 'term', 'label', 'created_by', 'created', 'modified')
    raw_id_fields = ('ontology', )
    search_fields = ('ontology__label', 'identifier', 'label', 'description', 'synonyms__description')
    save_as = True


class SynonymAdmin(admin.ModelAdmin):
    model = models.Synonym
    list_display = ('term', 'scope', 'created', 'modified')
    raw_id_fields = ('term', )
    list_filter = ('scope', )
    search_fields = ('description', )
    save_as = True


class CrossReferenceAdmin(admin.ModelAdmin):
    model = models.CrossReference
    list_display = ('term', 'source', 'source_value', 'created', 'modified')
    raw_id_fields = ('term', )
    list_filter = ('source', )
    search_fields = ('source', 'source_value', )
    save_as = True


class RelationshipAdmin(admin.ModelAdmin):
    model = models.Relationship
    list_display = ('type', 'term', 'related_term', 'created', 'modified')
    raw_id_fields = ('type', 'term', 'related_term')
    list_filter = ('type', )
    search_fields = (
        'term__label',
        'term__description',
        'term__synonyms__description',
        'related_term__label',
        'related_term__description',
        'related_term__synonyms__description'
    )
    save_as = True


class RelationshipTypeAdmin(admin.ModelAdmin):
    model = models.RelationshipType
    list_display = ('label', 'created', 'modified')
    search_fields = ('label', )
    save_as = True


admin.site.register(models.Ontology, OntologyAdmin)
admin.site.register(models.Term, TermAdmin)
admin.site.register(models.Synonym, SynonymAdmin)
admin.site.register(models.CrossReference, CrossReferenceAdmin)
admin.site.register(models.Relationship, RelationshipAdmin)
admin.site.register(models.RelationshipType, RelationshipTypeAdmin)
